/* $Id: ast_assignment.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_ASSIGNMENT_H
#define WTPARSER_AST_ASSIGNMENT_H

#include "../astnode.h"
#include "ast_statement.h"
#include <deque>
#include <string>

namespace wtast
{

class LValue;
class Expression;

class Assignment : public Statement
{
	typedef std::deque<LValue*> lvalues_t;
	lvalues_t   mLValues;
	Expression* mExpression;

public:
	Assignment(LValue* lval, Expression* expr)
		: Statement()
		, mExpression(expr)
	{
		if(lval)
		{
			AddFront(lval);
		}
	}

	~Assignment();

	void Accept(AstVisitor* vis);

	Assignment* Copy();

	Kind GetKind() const
	{
		return ASSIGNMENT;
	}

	std::size_t GetNumberOfLValues() const
	{
		return mLValues.size();
	}

	LValue* GetLValue(std::size_t idx)
	{
		if(idx<mLValues.size())
		{
			return mLValues[idx];
		}
		else
		{
			return 0;
		}
	}

	void AddFront(LValue* lval)
	{
		mLValues.push_front(lval);
	}

	void AddBack(LValue* lval)
	{
		mLValues.push_back(lval);
	}

	Expression* GetExpression()
	{
		return mExpression;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_ASSIGNMENT_H

