/* $Id: ast_enumtype.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_ENUMTYPE_H
#define WTPARSER_AST_ENUMTYPE_H

#include "../astnode.h"
#include "welltype/wtruntime.h"
#include "ast_type.h"
#include <deque>
#include <map>
#include <string>

namespace wtast
{

class Enumerator;

class EnumType : public Type
{
	typedef std::deque<Enumerator*> enumerators_t;
	typedef std::map<std::string, Enumerator*> enumeratormap_t;
	enumerators_t   mEnumerators;
	enumeratormap_t mEnumeratorMap;
	std::string     mName;
	wt_u32          mTypeId;
	wt_u32          mNextOrdinal;

public:
	EnumType(const std::string & name, wt_u32 type_id)
		: Type()
		, mName(name)
		, mTypeId(type_id)
		, mNextOrdinal(1)
	{
		// ok
	}

	~EnumType();

	void Accept(AstVisitor* vis);

	EnumType* Copy();

	Kind GetKind() const
	{
		return ENUMTYPE;
	}

	std::string GetName() const
	{
		return mName;
	}

	wt_u32 GetTypeId() const
	{
		return mTypeId;
	}

	wt_u32 GetLoadSize() const;

	bool IsObject() const
	{
		return false;
	}

	bool IsDefaultConstructible() const
	{
		return false;
	}

	bool HasFunctionType() const
	{
		return false;
	}

	bool IsCopyable() const
	{
		return true;
	}

	void AddEnumerator(Enumerator* enumerator);

	std::size_t GetNumberOfEnumerators() const
	{
		return mEnumerators.size();
	}

	Enumerator* GetEnumerator(std::size_t idx)
	{
		if(idx<mEnumerators.size())
		{
			return mEnumerators[idx];
		}
		else
		{
			return 0;
		}
	}

	Enumerator* GetEnumerator(const std::string & enumeratorname)
	{
		enumeratormap_t::iterator it = mEnumeratorMap.find(enumeratorname);

		if(mEnumeratorMap.end()!=it)
		{
			return it->second;
		}

		return 0;
	}

	void SetMutable()
	{
		mTypeId |= WTTF_MUTABLE;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_ENUMTYPE_H
