//================================================================================================
// $Id: builtins.h 6504 2023-04-03 17:46:07Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Builtin functions
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_BUILTINS_H
#define WTPARSER_BUILTINS_H

#include <string>
#include <vector>
#include "uniqueid.h"
#include "welltype/wtruntime.h"

namespace wtparser
{
namespace builtin
{

typedef std::vector<wt_u16> InstructionVec;
typedef std::vector<wt_u32> TypeVec;

class Builtins;

class Builtin
{
	std::string    mName;
	TypeVec        mArgs;
	TypeVec        mRets;
	bool           mPure;
	bool           mGen;
	InstructionVec mInstructions;
	UniqueId       mUniqueId;

	Builtin(const std::string & name, const TypeVec & args, const TypeVec & rets, bool pure, const InstructionVec & ins, UniqueId unique_id)
		: mName(name)
		, mArgs(args)
		, mRets(rets)
		, mPure(pure)
		, mGen(false)
		, mInstructions(ins)
		, mUniqueId(unique_id)
	{
		// ok
	}

	Builtin(const std::string & name, const TypeVec & args, const TypeVec & rets, bool pure, bool gen, const InstructionVec & ins, UniqueId unique_id)
		: mName(name)
		, mArgs(args)
		, mRets(rets)
		, mPure(pure)
		, mGen(gen)
		, mInstructions(ins)
		, mUniqueId(unique_id)
	{
		// ok
	}

public:
	const std::string &    GetName()     const { return mName;         }
	const TypeVec &        GetArgs()     const { return mArgs;         }
	const TypeVec &        GetRets()     const { return mRets;         }
	bool                   IsPure()      const { return mPure;         }
	bool                   IsGenerated() const { return mGen;          }
	const InstructionVec & GetInstrVec() const { return mInstructions; }
	UniqueId               GetUniqueId() const { return mUniqueId;     }

	friend class Builtins;
};

typedef std::vector<Builtin> BuiltinVec;

class Builtins
{
	BuiltinVec mBuiltins;
	UniqueId   mLastUniqueId;

public:
	Builtins();
	UniqueId GetLastUniqueId() const { return mLastUniqueId; }

	typedef BuiltinVec::const_iterator const_iterator;

	std::size_t    size()  const { return mBuiltins.size();  }
	const_iterator begin() const { return mBuiltins.begin(); }
	const_iterator end()   const { return mBuiltins.end();   }
};

extern const Builtins builtins;

} // builtin
} // wtparser

#endif // WTPARSER_BUILTINS_H
