#include "config_private.h"
#include "function.h"
#include "vm.h"
#include "funcsig.h"
#include "program.h"

struct wtr_function* wtr_function_new(struct wtr_vm* const vm,
	struct wtr_funcsig* sig, struct wtr_cfunction* cfunc,
	struct wtr_program* prog, wt_u32 pc)
{
	struct wtr_function* fh;

	if((cfunc && prog) || (!cfunc && !prog))
	{
		return NULL;
	}

	if(NULL!=wtr_vm_findfunction(vm, sig))
	{
		return NULL;
	}

	if(NULL!=(fh=(struct wtr_function*)wtr_vm_sys_malloc(vm,
		sizeof(struct wtr_function))))
	{
		fh->vm       = vm;
		fh->program  = prog;
		fh->cfunc    = cfunc;
		fh->funcsig  = sig;
		fh->refcount = 0;
		fh->hancount = 0;
		fh->pfn._pad = 0;

		if(NULL!=cfunc)
		{
			/* artificially increase the reference counter for C functions, */
			/* because these functions shall not be implicitly deleted */
			++fh->refcount;

			fh->pfn.ptr = cfunc;
			fh->pfn.ext = vm->external_tag;
		}
		else
		{
			fh->pfn.ptr = prog;
			fh->pfn.ext = prog->pc_base + pc;
		}

		wtr_list_insert_tail(&vm->list_functions, &fh->link_functions);
	}

	return fh;
}

void wtr_function_release(struct wtr_vm* const vm,
	struct wtr_function* const func)
{
	if(NULL!=func && (0==func->refcount || 0==(--func->refcount)))
	{
		wtr_list_remove(&func->link_functions);
		wtr_function_remhandle(vm, func);
		wtr_funcsig_delete(vm, func->funcsig);
		wtr_cfunction_delete(vm, func->cfunc);
		wtr_vm_sys_free(vm, func);
	}
}

wt_function wtr_function_genhandle(struct wtr_vm* const vm,
	struct wtr_function* const func)
{
	if(NULL!=func)
	{
		if(0==func->hancount)
		{
			++func->refcount;

			if(NULL!=func->program)
			{
				// TODO: check for program, but increments vm??
				++func->vm->refcount;
			}
		}

		++func->hancount;
	}

	return func;
}

void wtr_function_remhandle(struct wtr_vm* const vm, wt_function func)
{
	if(NULL!=func && (0==func->hancount || 0==(--func->hancount)))
	{
		if(NULL!=func->program)
		{
			// TODO: check for program, but decrements vm??
			--func->vm->refcount;
		}
	}
}

struct wtr_cfunction* wtr_cfunction_new(struct wtr_vm* const vm,
	struct wtr_funcsig* sig, WT_PFN_EXTFUNC func, void* token)
{
	struct wtr_cfunction* const cf =
		(struct wtr_cfunction*)wtr_vm_sys_malloc(vm,
			sizeof(struct wtr_cfunction));

	if(NULL!=cf)
	{
		cf->vm          = vm;
		cf->funcsig     = sig;
		cf->token       = token;
		cf->extfunc_pfn = func;

		++vm->refcount;
	}

	return cf;
}

void wtr_cfunction_delete(struct wtr_vm* const vm,
	struct wtr_cfunction* const cfunc)
{
	if(NULL!=cfunc)
	{
		--vm->refcount;

		wtr_vm_sys_free(vm, cfunc);
	}
}
